# -*- coding: utf-8 -*-
import json
import os
import os.path
import subprocess
import zc.buildout


class Recipe(object):
    def __init__(self, buildout, name, options):
        self.buildout, self.name, self.options = buildout, name, options

        if 'packages' not in options:
            raise zc.buildout.UserError('Missing packages option')

        parts_dir = buildout['buildout']['parts-directory']
        base_dir = os.path.join(parts_dir, 'bower')
        options.setdefault('base-directory', base_dir)
        options.setdefault('executable', 'bower')
        options.setdefault('downloads', 'downloads')

        # Remove unnecessary whitespace.
        packages = [p.strip() for p in options['packages'].splitlines()
                    if p.strip() != '']
        options['packages'] = ' '.join(packages)

    def install(self):
        return self.update()

    def update(self):
        base_dir = self.options['base-directory']
        download_dir = self.options['downloads']
        bowerrc = os.path.join(base_dir, '.bowerrc')

        if not os.path.exists(base_dir):
            os.makedirs(base_dir)

        with open(bowerrc, 'w') as f:
            json.dump({'directory': download_dir}, f)

        cmd = '{} install {}'.format(self.options['executable'],
                                     self.options['packages'])
        os.chdir(base_dir)
        subprocess.call(cmd, shell=True)

        return bowerrc, os.path.normpath(os.path.join(base_dir, download_dir))



from bowerrecipe import Recipe
import json
import mock
import os.path
import shutil
import tempfile
import unittest
import zc.buildout


@mock.patch('subprocess.call')
@mock.patch('os.chdir')
class RecipeTest(unittest.TestCase):

    def setUp(self):
        self.parts_dir = tempfile.mkdtemp()
        self.base_dir = os.path.join(self.parts_dir, 'bower')
        self.buildout = {'buildout': {
                             'parts-directory': self.parts_dir}}

    def tearDown(self):
        shutil.rmtree(self.parts_dir, ignore_errors=True)

    def test_exe_should_default_to_bower_on_path(self, chdir, spcall):
        options = {'recipe': 'bowerrecipe', 'packages': 'jquery'}
        Recipe(self.buildout, 'bower', options)
        self.assertEqual('bower', options['executable'])

    def test_at_least_one_package_should_be_specified(self, chdir, spcall):
        options = {'recipe': 'bowerrecipe'}
        self.assertRaises(zc.buildout.UserError, Recipe,
                          self.buildout, 'bower', options)

    def test_base_directory_should_have_a_default(self, chdir, spcall):
        options = {'recipe': 'bowerrecipe', 'packages': 'jquery'}
        Recipe(self.buildout, 'bower', options)
        self.assertEqual(self.base_dir, options['base-directory'])

    def test_bowerrc_file_should_reside_in_base_dir(self, chdir, spcall):
        options = {'recipe': 'bowerrecipe',
                   'base-directory': self.parts_dir,
                   'packages': 'jquery'}
        bowerrc = os.path.join(self.parts_dir, '.bowerrc')
        recipe = Recipe(self.buildout, 'bower', options)

        recipe.install()

        self.assertTrue(os.path.exists(bowerrc))

    def test_bowerrc_file_should_be_configured(self, chdir, spcall):
        options = {'recipe': 'bowerrecipe', 'packages': 'jquery'}
        recipe = Recipe(self.buildout, 'bower', options)
        expected = json.dumps({'directory': options['downloads']})
        bowerrc = os.path.join(self.base_dir, '.bowerrc')

        recipe.install()

        with open(bowerrc) as f:
            config = json.dumps(json.load(f))
        self.assertEqual(expected, config)

    def test_install_returns_bowerrc_and_downloads_dir(self, chdir, spcall):
        options = {'recipe': 'bowerrecipe',
                   'packages': 'jquery',
                   'downloads': '../downloads'}
        recipe = Recipe(self.buildout, 'bower', options)

        base_path, downloads_path = recipe.install()
        self.assertEqual(os.path.join(self.base_dir, '.bowerrc'), base_path)
        self.assertEqual(os.path.join(self.parts_dir, 'downloads'),
                         downloads_path)

    def test_install_should_call_bower_install(self, chdir, spcall):
        options = {'recipe': 'bowerrecipe',
                   'packages': 'jquery',
                   'executable': 'testbinary'}
        recipe = Recipe(self.buildout, 'bower', options)

        recipe.install()

        spcall.assert_called_with('testbinary install jquery', shell=True)

    def test_downloads_default_to_sensible_location(self, chdir, spcall):
        options = {'recipe': 'bowerrecipe', 'packages': 'jquery'}
        Recipe(self.buildout, 'bower', options)
        self.assertEqual('downloads', options['downloads'])

    def test_changes_to_correct_directory_pre_install(self, chdir, spcall):
        options = {'recipe': 'bowerrecipe', 'packages': 'jquery'}
        recipe = Recipe(self.buildout, 'bower', options)

        recipe.install()

        chdir.assert_called_with(self.base_dir)

    def test_multiple_packages_can_be_installed(self, chdir, spcall):
        options = {'recipe': 'bowerrecipe', 'packages': 'jquery bootstrap'}
        recipe = Recipe(self.buildout, 'bower', options)

        recipe.install()

        spcall.assert_called_with('bower install jquery bootstrap', shell=True)

    def test_packages_can_be_specified_on_multiple_lines(self, chdir, spcall):
        options = {'recipe': 'bowerrecipe',
                   'packages': '\n  jquery#1.8.1  \n  underscore  '}
        recipe = Recipe(self.buildout, 'bower', options)

        recipe.install()

        spcall.assert_called_with('bower install jquery#1.8.1 underscore',
                                  shell=True)



# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages


def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '0.2'

long_description = (
    read('README.rst')
    + '\n' +
    'Contributors\n'
    '============\n'
    + '\n' +
    read('AUTHORS')
    + '\n\n' +
    read('CHANGES.rst')
    + '\n\n' +
   'Download\n'
    '========\n')

entry_point = 'bowerrecipe:Recipe'
entry_points = {"zc.buildout": ["default = %s" % entry_point]}

tests_require = ['zc.buildout', 'mock']

setup(name='bowerrecipe',
      version=version,
      description="zc.buildout recipe to install static resources using Twitter Bower.",
      long_description=long_description,
      # Get more strings from
      # http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Framework :: Buildout',
        'Framework :: Buildout :: Recipe',
        'License :: OSI Approved :: MIT License',
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        ],
      keywords='',
      author='Anshuman Bhaduri',
      author_email='anshuman.bhaduri0@gmail.com',
      url='http://github.com/anshumanb/bowerrecipe',
      license='MIT',
      packages=find_packages(exclude=['ez_setup']),
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        'zc.buildout'],
      tests_require=tests_require,
      extras_require=dict(tests=tests_require),
      test_suite='bowerrecipe.tests.tests',
      entry_points=entry_points,
      )
